/* ARCADEmini simulator (unix, console)
 * version 1.0.1 date 2005-10-19
 * part of the ARCADEmini project - http[s]://arcademini.schuermans.info/
 * Copyright (C) 2003-2005 Stephan Maka <stephan@spaceboyz.net>
 *                     and 1stein <1stein@schuermans.info>
 * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
 */

#include <stdio.h>
#include <SDL/SDL.h>
#include <unistd.h>
#include <stdlib.h>

#include "am_dev.h"

#define pixel_size_x 2
#define pixel_size_y 2

Uint32 display_pixelfmt[0x10];

SDL_Surface *screen;

int display_init()
{
	int i;

	SDL_Init(SDL_INIT_VIDEO);
	screen = SDL_SetVideoMode(pixel_size_x * 26, pixel_size_y * 20, 32, SDL_HWSURFACE | SDL_DOUBLEBUF);
	if (!screen)
	{
		fprintf(stderr, "Unable to initialize SDL: %s\n", SDL_GetError());
		return -1;
	}

	for(i = 0; i < 0x10; i++)
		display_pixelfmt[i] = SDL_MapRGB(screen->format, 0x00, 0xff * i / 0x10, 0x00);

        return 0;
}

int display_poll()
{
	SDL_Event event;

	while(SDL_PollEvent(&event))
		if (event.type == SDL_QUIT)
			return 1; //request end
	return 0; //do not request end
}

void putpixel(SDL_Surface *surface, int x, int y, Uint32 pixel)
{
    int bpp = surface->format->BytesPerPixel;
        /* Here p is the address to the pixel we want to set */
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

    switch(bpp) {
        case 1:
        *p = pixel;
        break;

    case 2:
        *(Uint16 *)p = pixel;
        break;

    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
            p[0] = (pixel >> 16) & 0xff;
                p[1] = (pixel >> 8) & 0xff;
            p[2] = pixel & 0xff;
            } else {
                p[0] = pixel & 0xff;
            p[1] = (pixel >> 8) & 0xff;
                p[2] = (pixel >> 16) & 0xff;
        }
        break;

    case 4:
        *(Uint32 *)p = pixel;
        break;
    }
}

/*display an ARCADEmini picture*/
void display_picture( st_am_state * p_state )
{	
  unsigned char x, y, xx, yy;

  if (SDL_MUSTLOCK(screen))
	  SDL_LockSurface(screen);
  
  for( y = 0; y < 20; y++ )
  {
    for( yy = 0; yy < pixel_size_y; yy++ )
    {
      for( x = 0; x < 26; x++ )
      {
         for( xx = 0; xx < pixel_size_x; xx++ )
          putpixel(screen, (x * pixel_size_x) + xx, (y * pixel_size_y) + yy, display_pixelfmt[p_state->picture[y][x] & 0x0F]);
      }
    }
  }

  if (SDL_MUSTLOCK(screen))
	  SDL_UnlockSurface(screen);

  SDL_Flip(screen);
}

void display_exit( )
{
}

void display_final( )
{
	SDL_Quit();
}

